@@ -4,14 +4,22 @@ from datetime import datetime
 import copy
 import logging
 import socket
+import traceback
+import random
 
 import yaml
 import redcap
 
 import dax
-from dax import DAX_Settings
-from dax.launcher import BUILD_SUFFIX
-from dax import log
+from . import dax_tools_utils
+from . import DAX_Settings
+from .launcher import BUILD_SUFFIX
+from . import log
+from .errors import AutoProcessorError, DaxError
+from . import utilities
+
+# dax manager has 3 main classes: DaxManager has a DaxProjectSettingsManager
+# which is a collection of DaxProjectSettings.
 
 # TODO: archive old logs
 
@@ -23,6 +31,11 @@ DAX_SETTINGS = DAX_Settings()
 LOGGER = log.setup_debug_logger('manager', None)
 
 
+def project_from_settings(settings_file):
+    proj = settings_file.split('settings-')[1].split('.yaml')[0]
+    return proj
+
+
 def get_this_instance():
     # build the instance name
     this_host = socket.gethostname().split('.')[0]
@@ -30,19 +43,6 @@ def get_this_instance():
     return '{}@{}'.format(this_user, this_host)
 
 
-def clean_lockfiles():
-    lock_dir = os.path.join(DAX_SETTINGS.get_results_dir(), 'FlagFiles')
-    lock_list = os.listdir(lock_dir)
-
-    # Make full paths
-    lock_list = [os.path.join(lock_dir, f) for f in lock_list]
-
-    # Check each lock file
-    for file in lock_list:
-        LOGGER.debug('checking lock file:{}'.format(file))
-        check_lockfile(file)
-
-
 def check_lockfile(file):
     # Try to read host-PID from lockfile
     try:
@@ -57,7 +57,7 @@ def check_lockfile(file):
         if host != this_host:
             LOGGER.debug('different host, cannot check PID:{}', format(file))
         elif pid_exists(pid):
-            LOGGER.debug('host matches and PID still exists')
+            LOGGER.debug('host matches and PID exists:{}'.format(str(pid)))
         else:
             LOGGER.debug('host matches and PID not running, deleting lockfile')
             os.remove(file)
@@ -80,12 +80,9 @@ def pid_exists(pid):
         return True  # no error, we can send a signal to the process
 
 
-def is_locked(settings_path):
-    lockfile_prefix = os.path.splitext(os.path.basename(settings_path))[0]
-    flagfile = os.path.join(
-        DAX_SETTINGS.get_results_dir(),
-        'FlagFiles',
-        '{}_{}'.format(lockfile_prefix, BUILD_SUFFIX))
+def is_locked(settings_path, lock_dir):
+    _prefix = os.path.splitext(os.path.basename(settings_path))[0]
+    flagfile = os.path.join(lock_dir, '{}_{}'.format(_prefix, BUILD_SUFFIX))
 
     LOGGER.debug('checking for flag file:{}'.format(flagfile))
     return os.path.isfile(flagfile)
@@ -176,11 +173,63 @@ class DaxProjectSettingsManager(object):
         self._general_form = general_form
         self._local_dir = local_dir
         self._instance_settings = instance_settings
-
-        # Initialize redcap project
+        self.module_names = []
+        self.processor_names = []
+        self.rebuild_projects = []
+        self.records = {}
         self._redcap = redcap.Project(redcap_url, redcap_key)
 
-    def write_each(self):
+        # Initialize by loading from redcap project
+        self.load()
+
+    def list_settings_files(self):
+        slist = os.listdir(self._local_dir)
+
+        # Make full paths
+        slist = [os.path.join(self._local_dir, f) for f in slist]
+
+        # Only yaml files
+        slist = [f for f in slist if f.endswith('.yaml') and os.path.isfile(f)]
+
+        return slist
+
+    def _load_metadata(self):
+        self.metadata = self._redcap.metadata
+
+    def _load_records(self):
+        # Build the list of _complete fields from lists of modules/procesors
+        field_list = self._redcap.field_names + ['general_complete']
+
+        p = DaxProjectSettingsManager.PROC_PREFIX
+        s = '_complete'
+        field_list += [p + f + s for f in self.processor_names]
+
+        p = DaxProjectSettingsManager.MOD_PREFIX
+        s = '_complete'
+        field_list += [p + f + s for f in self.module_names]
+
+        self.records = self._redcap.export_records(
+            fields=field_list, raw_or_label='label')
+
+    def load(self):
+        self._load_module_names()
+        self._load_processor_names()
+        self._load_metadata()
+        self._load_records()
+        LOGGER.info('loaded {} records'.format(len(self.records)))
+
+    def get_record(self, project):
+        # Get the record from the dataframe
+        rec = None
+        for r in self.records:
+            if r['project_name'] == project:
+                rec = r
+                break
+
+        return rec
+
+    def update_each(self):
+        errors = []
         now = datetime.now()
 
         # First load the settings from our defaults project
@@ -189,14 +238,44 @@ class DaxProjectSettingsManager(object):
         for name in self.project_names():
             settings = copy.deepcopy(default_settings)
 
-            LOGGER.info('Loading project: ' + name)
-            project = self.load_project(settings, name)
-            settings.add_project(project)
+            try:
+                LOGGER.info('Loading project: ' + name)
+                project = self.load_project(settings, name)
+                settings.add_project(project)
 
-            # Write project file
-            filename = os.path.join(
-                self._local_dir, 'settings-{}.yaml'.format(name))
-            self.write_settings_file(filename, settings, now)
+                filename = os.path.join(
+                    self._local_dir, 'settings-{}.yaml'.format(name))
+
+                if self.settings_match(settings, filename):
+                    LOGGER.info('settings unchanged:{}'.format(filename))
+                else:
+                    # Write project file
+                    self.write_settings_file(filename, settings, now)
+
+                    # Append project name to our list of rebuild projects
+                    self.rebuild_projects.append(name)
+
+            except DaxManagerError as e:
+                err = 'error in project settings:project={}\n{}'.format(
+                    project, e)
+                LOGGER.info(err)
+                errors.append(err)
+
+        return errors
+
+    def settings_match(self, settings, filename):
+        if not os.path.exists(filename):
+            # No existing file so we it changed
+            return False
+
+        # now compare old to new
+        new_settings = settings.dump()
+        old_settings = self.load_settings_file(filename)
+        return (new_settings == old_settings)
+
+    def load_settings_file(self, filename):
+        settings = utilities.read_yaml(filename)
+        return settings
 
     def write_settings_file(self, filename, settings, timestamp):
         LOGGER.info('Writing settings to file:' + filename)
@@ -217,9 +296,13 @@ class DaxProjectSettingsManager(object):
         rec['processorlib'] = ins['main_processorlib']
         rec['modulelib'] = ins['main_modulelib']
         rec['singularity_imagedir'] = ins['main_singularityimagedir']
+        rec['resdir'] = ins['main_resdir']
+        rec['jobtemplate'] = ins['main_jobtemplate']
+        rec['admin_email'] = ins['main_adminemail']
         rec['attrs'] = {}
         rec['attrs']['queue_limit'] = int(ins['main_queuelimit'])
         rec['attrs']['job_email_options'] = ins['main_jobemailoptions']
+        rec['attrs']['job_rungroup'] = ins['main_rungroup']
         rec['attrs']['xnat_host'] = ins['main_xnathost']
 
         return rec
@@ -230,39 +313,92 @@ class DaxProjectSettingsManager(object):
 
         return settings
 
-    def load_module_names(self):
+    def _load_module_names(self):
         p = DaxProjectSettingsManager.MOD_PREFIX
-        return [x.split(p)[1] for x in self._redcap.forms if x.startswith(p)]
+        self.module_names = sorted([
+            x.split(p)[1] for x in self._redcap.forms if x.startswith(p)])
 
-    def load_processor_names(self):
+    def _load_processor_names(self):
         p = DaxProjectSettingsManager.PROC_PREFIX
-        return [x.split(p)[1] for x in self._redcap.forms if x.startswith(p)]
+        self.processor_names = sorted([
+            x.split(p)[1] for x in self._redcap.forms if x.startswith(p)])
 
-    def load_module_record(self, module, project):
+    def get_module_keys(self, module):
+        _m = self.metadata
         prefix = DaxProjectSettingsManager.MOD_PREFIX
         form = prefix + module
-        rc_rec = self._redcap.export_records(
-            forms=[form], records=[project])[0]
-        dax_rec = {'name': module}
 
-        # Find module prefix
-        key = [x for x in rc_rec.keys() if x.endswith('_file')]
-        if len(key) > 1:
+        # Get all the fields for the module's form
+        all_list = [f['field_name'] for f in _m if f['form_name'] == form]
+
+        # Find the args key for this module
+        args_list = [f for f in all_list if f.endswith('_args')]
+        if len(args_list) > 1:
             msg = 'multiple _file keys for module:{}'.format(module)
             raise DaxManagerError(msg)
-        elif len(key) == 0:
+        elif len(args_list) == 0:
             msg = 'no _file key for module:{}'.format(module)
             raise DaxManagerError(msg)
 
-        file_key = key[0]
-        key_prefix = file_key.split('_file')[0]
+        args_key = args_list[0]
 
-        # Get the filepath
+        # Find the file key for this module
+        file_list = [f for f in all_list if f.endswith('_file')]
+        if len(file_list) > 1:
+            msg = 'multiple _file keys for module:{}'.format(module)
+            raise DaxManagerError(msg)
+        elif len(file_list) == 0:
+            msg = 'no _file key for module:{}'.format(module)
+            raise DaxManagerError(msg)
+
+        file_key = file_list[0]
+
+        return (file_key, args_key)
+
+    def get_processor_keys(self, processor):
+        _m = self.metadata
+
+        # Get all the fields for the module's form
+        prefix = DaxProjectSettingsManager.PROC_PREFIX
+        form = prefix + processor
+        all_list = [f['field_name'] for f in _m if f['form_name'] == form]
+
+        # Find the args key for this module
+        args_list = [f for f in all_list if f.endswith('_args')]
+        if len(args_list) > 1:
+            msg = 'multiple _file keys for processor:{}'.format(processor)
+            raise DaxManagerError(msg)
+        elif len(args_list) == 0:
+            msg = 'no _file key for processor:{}'.format(processor)
+            raise DaxManagerError(msg)
+
+        args_key = args_list[0]
+
+        # Find the file key for this module
+        file_list = [f for f in all_list if f.endswith('_file')]
+        if len(file_list) > 1:
+            msg = 'multiple _file keys for processor:{}'.format(processor)
+            raise DaxManagerError(msg)
+        elif len(file_list) == 0:
+            msg = 'no _file key for processor:{}'.format(processor)
+            raise DaxManagerError(msg)
+
+        file_key = file_list[0]
+
+        return (file_key, args_key)
+
+    def load_module_record(self, module, project):
+        rc_rec = self.get_record(project)
+        dax_rec = {'name': module}
+
+        # Get the _file and _args field name for this module
+        file_key, args_key = self.get_module_keys(module)
+
+        # Get the filepath using the file key
         dax_rec['filepath'] = rc_rec[file_key]
 
         # Parse arguments
-        args_key = key_prefix + '_args'
-        if rc_rec[args_key] and len(rc_rec[args_key]) > 0:
+        if args_key in rc_rec and len(rc_rec[args_key].strip()) > 0:
             rlist = rc_rec[args_key].strip().split('\r\n')
             rdict = {}
             for arg in rlist:
@@ -274,34 +410,26 @@ class DaxProjectSettingsManager(object):
         return dax_rec
 
     def load_processor_record(self, processor, project):
-        prefix = DaxProjectSettingsManager.PROC_PREFIX
-        form = prefix + processor
-        rc_rec = self._redcap.export_records(
-            forms=[form], records=[project])[0]
+        rc_rec = self.get_record(project)
         dax_rec = {'name': processor}
 
-        # Find processor prefix
-        key = [x for x in rc_rec.keys() if x.endswith('_file')]
-        if len(key) > 1:
-            msg = 'multiple _file keys for proc:{}'.format(processor)
-            raise DaxManagerError(msg)
-        elif len(key) == 0:
-            msg = 'no _file key found for proc:{}'.format(processor)
-            raise DaxManagerError(msg)
-
-        file_key = key[0]
-        key_prefix = file_key.split('_file')[0]
+        # Get the _file and _args field name for this processor
+        file_key, args_key = self.get_processor_keys(processor)
 
         # Get the filepath
         dax_rec['filepath'] = rc_rec[file_key]
 
         # Check for arguments
-        if rc_rec[key_prefix + '_args']:
-            rlist = rc_rec[key_prefix + '_args'].strip().split('\r\n')
+        if args_key in rc_rec and len(rc_rec[args_key].strip()) > 0:
+            rlist = rc_rec[args_key].strip().split('\r\n')
             rdict = {}
             for arg in rlist:
-                key, val = arg.split(':', 1)
-                rdict[key] = val.strip()
+                try:
+                    key, val = arg.split(':', 1)
+                    rdict[key] = val.strip()
+                except ValueError as e:
+                    msg = 'invalid arguments:{}'.format(e)
+                    raise DaxManagerError(msg)
 
             dax_rec['arguments'] = rdict
 
@@ -310,41 +438,36 @@ class DaxProjectSettingsManager(object):
     def is_enabled_module(self, module, project):
         prefix = DaxProjectSettingsManager.MOD_PREFIX
         form = prefix + module
-        rec = self._redcap.export_records(forms=[form], records=[project])[0]
+        rec = self.get_record(project)
         complete = rec[form + '_complete']
-        return (complete == '2')
+        return (complete == 'Complete')
 
     def is_enabled_processor(self, processor, project):
         prefix = DaxProjectSettingsManager.PROC_PREFIX
         form = prefix + processor
-        rec = self._redcap.export_records(forms=[form], records=[project])[0]
+        rec = self.get_record(project)
         complete = rec[form + '_complete']
-        return (complete == '2')
+        return (complete == 'Complete')
 
     def project_names(self):
         complete_field = self._general_form + '_complete'
         instance_field = 'gen_daxinstance'
-        name_field = 'project_name'
 
-        # Get projects from REDCap
-        plist = self._redcap.export_records(
-            fields=[name_field, instance_field, complete_field],
-            raw_or_label='label')
-
-        # Filter to only include projects for this instance
+        # Filter to only include projects for this instance that are Complete
         this_instance = get_this_instance()
-        plist = [x for x in plist if x[instance_field] == this_instance]
+        plist = [
+            r['project_name'] for r in self.records if
+            r[instance_field] == this_instance and
+            r[complete_field] == 'Complete']
 
-        # Return project names that are enabled
-        return [x[name_field] for x in plist if x[complete_field] == 'Complete']
+        return plist
 
     def load_project(self, settings, project):
         proj_proc = []
         proj_mod = []
 
         # Get the project modules
-        mod_names = self.load_module_names()
-        for name in mod_names:
+        for name in self.module_names:
             if not self.is_enabled_module(name, project):
                 continue
 
@@ -352,15 +475,14 @@ class DaxProjectSettingsManager(object):
             mod = self.load_module_record(name, project)
             mod['name'] = name
 
-            # Add the custom module to our settings
+            # Add the module to our settings
             settings.add_module(mod)
 
             # Append it to list for this project
             proj_mod.append(name)
 
         # Get the project processors
-        proc_names = self.load_processor_names()
-        for name in proc_names:
+        for name in self.processor_names:
             if not self.is_enabled_processor(name, project):
                 continue
 
@@ -376,46 +498,40 @@ class DaxProjectSettingsManager(object):
 
         return {
             'project': project,
-            'modules': ','.join(proj_mod),
-            'yamlprocessors': ','.join(proj_proc)}
+            'modules': ','.join(sorted(proj_mod)),
+            'yamlprocessors': ','.join(sorted(proj_proc))}
 
     def delete_disabled(self):
-        # Get disabled project names from REDCap
-        field = self._general_form + '_complete'
-        rlist = self._redcap.export_records(fields=['project_name', field])
-        disabled_list = [x['project_name'] for x in rlist if x[field] != '2']
-
         # Delete disabled project settings files
-        for name in disabled_list:
-            filename = os.path.join(
-                self._local_dir, 'settings-{}.yaml'.format(name))
-            if os.path.exists(filename):
-                LOGGER.info('deleting disabled project:{}'.format(filename))
-                os.remove(filename)
+        enabled_projects = self.project_names()
+        for curf in self.list_settings_files():
+            curp = project_from_settings(curf)
+            if curp not in enabled_projects:
+                LOGGER.info('deleting disabled project:{}'.format(curf))
+                os.remove(curf)
 
         return
 
     def get_last_start_time(self, project):
-        rec = self._redcap.export_records(
-            fields=['build_laststarttime'],
-            records=[project])[0]
+        rec = self.get_record(project)
 
         return rec['build_laststarttime']
 
     def get_last_run(self, project):
-        rec = self._redcap.export_records(
-            fields=[
-                'build_lastcompletestarttime', 'build_lastcompletefinishtime'],
-            records=[project])[0]
+        if project in self.rebuild_projects:
+            return None
 
+        # Find the record for this project
+        rec = self.get_record(project)
+
+        # Get the start/finish times of last complete build
         last_start = rec['build_lastcompletestarttime']
         last_finish = rec['build_lastcompletefinishtime']
-        last_run = None
 
         if last_start != '' and last_finish != '' and last_start < last_finish:
-            last_run = datetime.strptime(last_start, self.RCDATEFORMAT)
-
-        return last_run
+            return datetime.strptime(last_start, self.RCDATEFORMAT)
+        else:
+            return None
 
     def set_last_build_start(self, project):
         last_start = datetime.strftime(datetime.now(), self.RCDATEFORMAT)
@@ -432,9 +548,13 @@ class DaxProjectSettingsManager(object):
         try:
             response = self._redcap.import_records([rec])
             assert 'count' in response
-        except AssertionError as err:
+        except Exception as e:
             err = 'redcap import failed'
             LOGGER.info(err)
+            LOGGER.info(e)
+        except AssertionError as e:
+            err = 'redcap import failed'
+            LOGGER.info(e)
             raise DaxManagerError(err)
         except Exception as e:
             err = 'connection to REDCap interrupted'
@@ -492,64 +612,79 @@ class DaxManager(object):
 
     def __init__(self, api_url, api_key_instances, api_key_projects):
 
+        # TODO: test the api keys or catch errors from pycap to
+        # handle when redcap is down
+
         # Load settings for this instance
-        self.instance_settings = self.load_instance_settings(
+        instance_settings = self.load_instance_settings(
             api_url, api_key_instances)
 
-        LOGGER.debug(self.instance_settings)
+        LOGGER.debug(instance_settings)
 
-        self.settings_dir = self.instance_settings['main_projectsettingsdir']
-        self.log_dir = self.instance_settings['main_logdir']
-        self.max_build_count = int(self.instance_settings['main_buildlimit'])
+        self.settings_dir = instance_settings['main_projectsettingsdir']
+        self.log_dir = instance_settings['main_logdir']
+        self.max_build_count = int(instance_settings['main_buildlimit'])
+        self.max_upload_count = int(instance_settings['main_uploadlimit'])
+        self.res_dir = instance_settings['main_resdir']
+        self.admin_emails = instance_settings['main_adminemail'].split(',')
+        self.lock_dir = os.path.join(self.res_dir, 'FlagFiles')
+        self.job_template = instance_settings['main_jobtemplate']
+        self.smtp_host = instance_settings['main_smtphost']
+        self.mode = instance_settings['main_mode']
+        self.enabled = (instance_settings['main_complete'] == 'Complete')
 
         # Create our settings manager and update our settings directory
         self.settings_manager = DaxProjectSettingsManager(
-            api_url, api_key_projects,
-            self.instance_settings, self.settings_dir)
+            api_url,
+            api_key_projects,
+            instance_settings,
+            self.settings_dir)
 
-        self.refresh_settings()
+    def is_enabled_instance(self):
+        return (self.enabled)
+
+    def is_enabled_build(self):
+        return ('build' in self.mode.lower())
+
+    def is_enabled_launch(self):
+        return ('launch' in self.mode.lower())
+
+    def is_enabled_upload(self):
+        return ('upload' in self.mode.lower())
+
+    def is_enabled_update(self):
+        # update is tied to upload here
+        return ('upload' in self.mode.lower())
 
     def load_instance_settings(
             self, redcap_url, redcap_key, main_form='main'):
 
         self._main_form = main_form
 
-        # Initialize redcap project
+        # Initialize redcap projec
         self._redcap = redcap.Project(redcap_url, redcap_key)
 
         # get this instance name
         instance_name = get_this_instance()
         LOGGER.debug('instance={}'.format(instance_name))
 
-        # Return the reocrd associate with this instance_name
-        return self._redcap.export_records(records=[instance_name])[0]
+        # Return the record associated with this instance_name
+        fields = self._redcap.field_names + [main_form + '_complete']
+        return self._redcap.export_records(
+            records=[instance_name], fields=fields, raw_or_label='label')[0]
 
     def refresh_settings(self):
-        # Delete existing settings files
-        for filename in self.list_settings_files(self.settings_dir):
-            os.remove(filename)
+        # Update settings files, only writing if something changed
+        errors = self.settings_manager.update_each()
 
-        # Write settings files
-        self.settings_manager.write_each()
+        # Delete any left over settings files
+        self.settings_manager.delete_disabled()
 
         # Load settings files
-        self.settings_list = self.list_settings_files(self.settings_dir)
+        self.settings_list = self.settings_manager.list_settings_files()
         LOGGER.info(self.settings_list)
 
-    def list_settings_files(self, settings_dir):
-        slist = os.listdir(settings_dir)
-
-        # Make full paths
-        slist = [os.path.join(settings_dir, f) for f in slist]
-
-        # Only yaml files
-        slist = [f for f in slist if f.endswith('.yaml') and os.path.isfile(f)]
-
-        return slist
-
-    def project_from_settings(self, settings_file):
-        proj = settings_file.split('settings-')[1].split('.yaml')[0]
-        return proj
+        return errors
 
     def log_name(self, runtype, project, timestamp):
         dname = datetime.strftime(timestamp, self.DDATEFORMAT)
@@ -562,16 +697,16 @@ class DaxManager(object):
 
         return log
 
-    def queue_builds(self, build_pool, settings_list):
+    def queue_builds(self, build_pool):
         # TODO: sort builds by how long we expect them to take,
         # shortest to longest
 
         # Array to store result accessors
-        build_results = [None] * len(settings_list)
+        build_results = [None] * len(self.settings_list)
 
         # Run each
-        for i, settings_path in enumerate(settings_list):
-            proj = self.project_from_settings(settings_path)
+        for i, settings_path in enumerate(self.settings_list):
+            proj = project_from_settings(settings_path)
             log = self.log_name('build', proj, datetime.now())
             last_run = self.get_last_run(proj)
 
@@ -585,78 +720,120 @@ class DaxManager(object):
         return build_results
 
     def run(self):
+        # Refresh project settings
+        settings_errors = self.refresh_settings()
+
+        run_errors = settings_errors
         max_build_count = self.max_build_count
+        build_pool = None
+        build_results = None
+        num_build_threads = 0
 
-        # Build
-        lock_dir = os.path.join(DAX_SETTINGS.get_results_dir(), 'FlagFiles')
-        lock_list = os.listdir(lock_dir)
-        lock_list = [x for x in lock_list if x.endswith('_BUILD_RUNNING.txt')]
-        cur_build_count = len(lock_list)
-        LOGGER.info('count of already running builds:' + str(cur_build_count))
+        if self.is_enabled_build():
+            # Build
+            lock_list = os.listdir(self.lock_dir)
+            lock_list = [x for x in lock_list if x.endswith('_BUILD_RUNNING.txt')]
+            cur_build_count = len(lock_list)
+            LOGGER.info('count of already running builds:' + str(cur_build_count))
 
-        num_build_threads = max_build_count - cur_build_count
-        if num_build_threads < 1:
-            LOGGER.info('max builds already:{}'.format(str(cur_build_count)))
+            num_build_threads = max_build_count - cur_build_count
+            if num_build_threads < 1:
+                LOGGER.info('max builds already:{}'.format(str(cur_build_count)))
+            else:
+                LOGGER.info('starting {} more builds'.format(
+                    str(num_build_threads)))
+                build_pool = Pool(processes=num_build_threads)
+                build_results = self.queue_builds(build_pool)
+                build_pool.close()  # Close the pool, I dunno if this matters
+
+        if self.is_enabled_update():
+            # Update
+            LOGGER.info('updating')
+            for settings_path in self.settings_list:
+                try:
+                    proj = project_from_settings(settings_path)
+
+                    LOGGER.info('updating jobs:' + proj)
+                    log = self.log_name('update', proj, datetime.now())
+                    self.run_update(settings_path, log)
+                except (AutoProcessorError, DaxError) as e:
+                    err = 'error running update:project={}\n{}'.format(proj, e)
+                    LOGGER.error(err)
+                    run_errors.append(err)
+
+        if self.is_enabled_launch():
+            # Launch - report to log if locked
+            LOGGER.info('launching')
+            # TODO: implement a better sorting method here so that launching
+            # is explicitly fair. This random sample is a temporary solution.
+            for settings_path in random.sample(
+                    self.settings_list, len(self.settings_list)):
+                try:
+                    proj = project_from_settings(settings_path)
+
+                    LOGGER.info('launching jobs:' + proj)
+                    log = self.log_name('launch', proj, datetime.now())
+                    self.run_launch(settings_path, log)
+                except (AutoProcessorError, DaxError) as e:
+                    err = 'error running launch:project={}\n{}'.format(proj, e)
+                    LOGGER.error(err)
+                    run_errors.append(err)
+
+        if self.is_enabled_upload():
+            # Upload - report to log if locked
+            log = self.log_name('upload', 'upload', datetime.now())
+            upload_process = Process(
+                target=self.run_upload,
+                args=(log,))
+            LOGGER.info('starting upload')
+            upload_process.start()
+            LOGGER.info('waiting for upload')
+            upload_process.join()
+            LOGGER.info('upload complete')
+
+        if self.is_enabled_build() and num_build_threads > 0:
+            # Wait for builds to finish
+            LOGGER.info('waiting for builds to finish')
+            build_pool.join()
+
+            # Extract any errors and add to list
+            build_errors = [x.get() for x in build_results if x.get()]
+            run_errors.extend(build_errors)
+
+        if run_errors:
+            LOGGER.info('ERROR:dax manager DONE with errors')
         else:
-            LOGGER.info('starting {} more builds'.format(str(num_build_threads)))
-            build_pool = Pool(processes=num_build_threads)
-            build_results = self.queue_builds(build_pool, self.settings_list)
-            build_pool.close()  # Close the pool, I dunno if this matters
+            LOGGER.info('run DONE with no errors!')
 
-        # Update
-        LOGGER.info('updating')
-        for settings_path in self.settings_list:
-            proj = self.project_from_settings(settings_path)
-
-            LOGGER.info('updating jobs:' + proj)
-            log = self.log_name('update', proj, datetime.now())
-            self.run_update(settings_path, log)
-
-        # Launch - report to log if locked
-        LOGGER.info('launching')
-        for settings_path in self.settings_list:
-            proj = self.project_from_settings(settings_path)
-
-            LOGGER.info('launching jobs:' + proj)
-            log = self.log_name('launch', proj, datetime.now())
-            self.run_launch(settings_path, log)
-
-        # Upload - report to log if locked
-        log = self.log_name('upload', 'upload', datetime.now())
-        upload_process = Process(
-            target=self.run_upload,
-            args=(log,))
-        LOGGER.info('starting upload')
-        upload_process.start()
-        LOGGER.info('waiting for upload')
-        upload_process.join()
-        LOGGER.info('upload complete')
-
-        # Wait for builds to finish
-        LOGGER.info('waiting for builds to finish')
-        build_pool.join()
+        return run_errors
 
     def run_build(self, project, settings_file, log_file, lastrun):
+        build_error = None
+
         # Check for existing lock
-        if is_locked(settings_file):
+        if is_locked(settings_file, self.lock_dir):
             LOGGER.warn('cannot build, lock exists:{}'.format(settings_file))
             # TODO: check if it's really running, if not send a notification
         else:
             # dax.bin.build expects a map of project to lastrun
             proj_lastrun = {project: lastrun}
 
-            LOGGER.info('run_build:{},{}'.format(project, lastrun))
+            LOGGER.info('run_build:start:{},{}'.format(project, lastrun))
             self.set_last_build_start(project)
             logging.getLogger('dax').handlers = []
-            dax.bin.build(
-                settings_file, log_file, debug=True, proj_lastrun=proj_lastrun)
-            logging.getLogger('dax').handlers = []
-
-            # TODO: check for errors in log file and set to RED if any found,
-            # also could upload last log file
+            try:
+                dax.bin.build(
+                    settings_file, log_file, True, proj_lastrun=proj_lastrun)
+                logging.getLogger('dax').handlers = []
+            except Exception:
+                err = 'error running build:proj={}:err={}'.format(
+                    project, traceback.format_exc())
+                LOGGER.error(err)
+                build_error = err
 
             self.set_last_build_complete(project)
             LOGGER.info('run_build:done:{}'.format(project))
+            return build_error
 
     def set_last_build_start(self, project):
         self.settings_manager.set_last_build_start(project)
@@ -679,7 +856,8 @@ class DaxManager(object):
 
     def run_upload(self, log_file):
         logging.getLogger('dax').handlers = []
-        dax.dax_tools_utils.upload_tasks(log_file, True)
+        dax_tools_utils.upload_tasks(log_file, debug=True, resdir=self.res_dir,
+                                     num_threads=self.max_upload_count)
         logging.getLogger('dax').handlers = []
 
     def all_ready(self, results):
@@ -691,19 +869,49 @@ class DaxManager(object):
 
         return ready
 
+    def email_errors(self, errors):
+        # email the errors
+        _msg = 'ERRORS:\n\n'
+        _msg += '\n\n'.join(errors)
+        _msg += '\n\n'
+        _to = self.admin_emails
+        _subj = 'ERROR:dax manager'
+        utilities.send_email_netrc(self.smtp_host, _to, _subj, _msg)
+
+    def clean_lockfiles(self):
+        lock_list = os.listdir(self.lock_dir)
+
+        # Make full paths
+        lock_list = [os.path.join(self.lock_dir, f) for f in lock_list]
+
+        # Check each lock file
+        for file in lock_list:
+            LOGGER.debug('checking lock file:{}'.format(file))
+            check_lockfile(file)
+
 
 if __name__ == '__main__':
     API_URL = os.environ['API_URL']
     API_KEY_P = os.environ['API_KEY_DAX_PROJECTS']
     API_KEY_I = os.environ['API_KEY_DAX_INSTANCES']
 
-    # Clean up existing lock files
-    clean_lockfiles()
-
     # Make our dax manager
     manager = DaxManager(API_URL, API_KEY_I, API_KEY_P)
 
-    # And run it
-    manager.run()
+    if manager.is_enabled_instance():
+        # Clean up existing lock files
+        manager.clean_lockfiles()
+
+        # And run it
+        errors = manager.run()
+
+        # Show errors
+        if errors:
+            msg = 'ERRORS:\n'
+            msg += '\n\n'.join(errors)
+            msg += '\n\n'
+            LOGGER.info(msg)
+    else:
+        LOGGER.info('this instance is currently disabled in REDCap.')
 
     LOGGER.info('ALL DONE!')
