@@ -51,6 +51,14 @@ def str_to_timedelta(delta_str):
         raise ValueError('invalid timedelta string value')
 
 
+def check_res_dir(resdir):
+    check_dir(os.path.join(resdir, 'DISKQ'))
+    check_dir(os.path.join(os.path.join(resdir, 'DISKQ'), 'INPUTS'))
+    check_dir(os.path.join(os.path.join(resdir, 'DISKQ'), 'OUTLOG'))
+    check_dir(os.path.join(os.path.join(resdir, 'DISKQ'), 'BATCH'))
+    check_dir(os.path.join(resdir, 'FlagFiles'))
+
+
 def check_dir(dir_path):
     try:
         os.makedirs(dir_path)
@@ -73,15 +81,19 @@ class Launcher(object):
     """ Launcher object to manage a list of projects from a settings file """
 
     def __init__(self,
+                 resdir,
                  project_process_dict=dict(),
                  project_modules_dict=dict(),
                  yaml_dict=dict(),
                  priority_project=None,
-                 queue_limit=DAX_SETTINGS.get_queue_limit(),
-                 root_job_dir=DAX_SETTINGS.get_root_job_dir(),
+                 queue_limit='10',
+                 root_job_dir='/tmp',
                  xnat_user=None, xnat_pass=None, xnat_host=None, cr=None,
-                 job_email=None, job_email_options='bae', max_age=7,
-                 launcher_type=DAX_SETTINGS.get_launcher_type()):
+                 job_email=None, job_email_options='FAIL', job_rungroup=None,
+                 launcher_type='diskq-combined',
+                 job_template='~/job_template.txt',
+                 smtp_host=None,
+                 timeout_emails=None):
 
         """
         Entry point for the Launcher class
@@ -98,11 +110,15 @@ class Launcher(object):
         :param xnat_pass: XNAT Password. By default, use env variable.
         :param job_email: job email address for report
         :param job_email_options: email options for the jobs
+        :param job_rungroup: cluster group to run the job under
         :param max_age: maximum time before updating again a session
         :return: None
         """
         self.queue_limit = queue_limit
         self.root_job_dir = root_job_dir
+        self.resdir = resdir
+        self.smtp_host = smtp_host
+        self.timeout_emails = timeout_emails
 
         # Processors:
         if not isinstance(project_process_dict, dict):
@@ -160,16 +176,12 @@ name as a key and list of yaml filepaths as values.'
 
         self.job_email = job_email
         self.job_email_options = job_email_options
-        self.max_age = DAX_SETTINGS.get_max_age()
+        self.job_rungroup = job_rungroup
         self.launcher_type = launcher_type
+        self.job_template = job_template
 
-        # Creating Folders for flagfile/pbs/outlog in RESULTS_DIR
-        res_dir = DAX_SETTINGS.get_results_dir()
-        check_dir(os.path.join(res_dir, 'DISKQ'))
-        check_dir(os.path.join(os.path.join(res_dir, 'DISKQ'), 'INPUTS'))
-        check_dir(os.path.join(os.path.join(res_dir, 'DISKQ'), 'OUTLOG'))
-        check_dir(os.path.join(os.path.join(res_dir, 'DISKQ'), 'BATCH'))
-        check_dir(os.path.join(res_dir, 'FlagFiles'))
+        # Create Folders for flagfile/pbs/outlog in RESULTS_DIR
+        check_res_dir(resdir)
 
         self.xnat_host = xnat_host
         if not self.xnat_host:
@@ -223,8 +235,7 @@ name as a key and list of yaml filepaths as values.'
         LOGGER.info('-------------- Launch Tasks --------------')
         LOGGER.info('launcher_type = %s' % self.launcher_type)
 
-        res_dir = DAX_SETTINGS.get_results_dir()
-        flagfile = os.path.join(os.path.join(res_dir, 'FlagFiles'),
+        flagfile = os.path.join(os.path.join(self.resdir, 'FlagFiles'),
                                 '%s_%s' % (lockfile_prefix, LAUNCH_SUFFIX))
 
         project_list = self.init_script(flagfile, project_local,
@@ -234,10 +245,12 @@ name as a key and list of yaml filepaths as values.'
             LOGGER.info('no projects to launch')
         else:
             msg = 'Loading task queue from: %s'
-            LOGGER.info(msg % os.path.join(res_dir, 'DISKQ'))
+            LOGGER.info(msg % os.path.join(self.resdir, 'DISKQ'))
             task_list = load_task_queue(
+                self.resdir,
                 status=task.NEED_TO_RUN,
-                proj_filter=project_list)
+                proj_filter=project_list,
+                sess_filter=sessions_local)
 
             msg = '%s tasks that need to be launched found'
             LOGGER.info(msg % str(len(task_list)))
@@ -267,6 +280,9 @@ name as a key and list of yaml filepaths as values.'
         :param force_no_qsub: run the job locally on the computer (serial mode)
         :return: None
         """
+
+        # TODO: determine if writeonly still does anything?
+
         if force_no_qsub:
             LOGGER.info('No qsub - Running job locally on your computer.')
         else:
@@ -332,8 +348,7 @@ cluster queue"
         LOGGER.info('-------------- Update Tasks --------------')
         LOGGER.info('launcher_type = %s' % self.launcher_type)
 
-        res_dir = DAX_SETTINGS.get_results_dir()
-        flagfile = os.path.join(os.path.join(res_dir, 'FlagFiles'),
+        flagfile = os.path.join(os.path.join(self.resdir, 'FlagFiles'),
                                 '%s_%s' % (lockfile_prefix, UPDATE_SUFFIX))
         project_list = self.init_script(flagfile, project_local,
                                         type_update=2, start_end=1)
@@ -342,8 +357,8 @@ cluster queue"
             LOGGER.info('no projects to launch')
         else:
             msg = 'Loading task queue from: %s'
-            LOGGER.info(msg % os.path.join(res_dir, 'DISKQ'))
-            task_list = load_task_queue(proj_filter=project_list)
+            LOGGER.info(msg % os.path.join(self.resdir, 'DISKQ'))
+            task_list = load_task_queue(self.resdir, proj_filter=project_list)
 
             LOGGER.info('%s tasks found.' % str(len(task_list)))
 
@@ -388,15 +403,19 @@ cluster queue"
         LOGGER.info('launcher_type = %s' % self.launcher_type)
         LOGGER.info('mod delta = %s' % str(mod_delta))
 
-        res_dir = DAX_SETTINGS.get_results_dir()
-        flagfile = os.path.join(os.path.join(res_dir, 'FlagFiles'),
+        flagfile = os.path.join(os.path.join(self.resdir, 'FlagFiles'),
                                 '%s_%s' % (lockfile_prefix, BUILD_SUFFIX))
         project_list = self.init_script(flagfile, project_local,
                                         type_update=1, start_end=1)
 
         LOGGER.info('Connecting to XNAT at %s' % self.xnat_host)
-        with XnatUtils.get_interface(self.xnat_host, self.xnat_user,
-                                     self.xnat_pass) as intf:
+        with XnatUtils.get_interface(
+            self.xnat_host,
+            self.xnat_user,
+            self.xnat_pass,
+            self.smtp_host,
+            self.timeout_emails
+        ) as intf:
 
             if not XnatUtils.has_dax_datatypes(intf):
                 err = 'error: dax datatypes are not installed on xnat <%s>'
@@ -636,7 +655,6 @@ cluster queue"
             init_timestamp, final_timestamp))
         if final_timestamp > init_timestamp:
             res_obj = csess.full_object().resource('BUILD_LOGS')
-            print(res_obj)
             LOGGER.debug('uploading session log:' + tmp_file)
             XnatUtils.upload_file_to_obj(tmp_file, res_obj)
         else:
@@ -705,14 +723,23 @@ in session %s'
         :return: None
         """
         # sess_info = csess.info()
-        res_dir = DAX_SETTINGS.get_results_dir()
         xnat_session = csess.full_object()
 
         for auto_proc in auto_proc_list:
             # return a mapping between the assessor input sets and existing
             # assessors that map to those input sets
-            auto_proc.parse_session(csess, sessions)
-            mapping = auto_proc.get_assessor_mapping()
+
+            # BDB 6/5/21
+            # For now, we will not process pet sessions here, but will
+            # include them as additional params for mr sessions.
+            if csess.datatype() == 'xnat:petSessionData':
+                LOGGER.debug('pet session, skipping auto processors')
+                continue
+            else:
+                pets = [x for x in sessions if x.datatype() == 'xnat:petSessionData']
+                mrs = [x for x in sessions if x.datatype() != 'xnat:petSessionData']
+
+            mapping = auto_proc.parse_session(csess, mrs, pets)
 
             if mapping is None:
                 continue
@@ -729,6 +756,7 @@ in session %s'
                 else:
                     assessors = []
                     for p in p_assrs:
+                        # BDB 6/5/21 is there ever more than one assessor here?
                         info = p.info()
                         procstatus = info['procstatus']
                         qcstatus = info['qcstatus']
@@ -739,8 +767,8 @@ in session %s'
                     procstatus = assessor[2]
                     qcstatus = assessor[3]
                     if task_needs_to_run(procstatus, qcstatus):
-                        xtask = XnatTask(auto_proc, assessor[0], res_dir,
-                                         os.path.join(res_dir, 'DISKQ'))
+                        xtask = XnatTask(auto_proc, assessor[0], self.resdir,
+                                         os.path.join(self.resdir, 'DISKQ'))
 
                         if task_needs_status_update(qcstatus):
                             xtask.update_status()
@@ -750,7 +778,8 @@ in session %s'
                             assessor[0], sessions,
                             self.root_job_dir,
                             self.job_email,
-                            self.job_email_options)
+                            self.job_email_options,
+                            self.job_rungroup)
                         deg = 'proc_status=%s, qc_status=%s'
                         LOGGER.debug(deg % (proc_status, qc_status))
 
@@ -934,8 +963,7 @@ The project is not part of the settings."""
 
         # Get lists of processors for this project
         pp_dict = self.project_process_dict.get(project_id, None)
-        auto_procs =\
-            processors.processors_by_type(pp_dict)
+        auto_procs = processors.processors_by_type(pp_dict)
 
         # Get lists of assessors for this project
         assr_list = self.get_assessors_list(xnat, project_id, sessions_local)
@@ -989,7 +1017,7 @@ The project is not part of the settings."""
                                         assr_info['subject_id'],
                                         assr_info['session_id'],
                                         assr_info['ID'])
-            cur_task = Task(task_proc, assr, DAX_SETTINGS.get_results_dir())
+            cur_task = Task(task_proc, assr, self.resdir)
             return cur_task
 
     @staticmethod
@@ -1075,22 +1103,24 @@ The project is not part of the settings."""
         return len(proc_types.difference(assr_types)) > 0
 
 
-def load_task_queue(status=None, proj_filter=None):
+def load_task_queue(resdir, status=None, proj_filter=None, sess_filter=None):
     """ Load the task queue for DiskQ"""
     task_list = list()
-    diskq_dir = os.path.join(DAX_SETTINGS.get_results_dir(), 'DISKQ')
-    results_dir = DAX_SETTINGS.get_results_dir()
+    diskq_dir = os.path.join(resdir, 'DISKQ')
 
     for t in os.listdir(os.path.join(diskq_dir, 'BATCH')):
-        # TODO:complete filtering by project/subject/session/type
-        if proj_filter:
+        if proj_filter or sess_filter:
             assr = XnatUtils.AssessorHandler(t)
-            if assr.get_project_id() not in proj_filter:
+            if proj_filter and assr.get_project_id() not in proj_filter:
+                LOGGER.debug('ignoring:' + t)
+                continue
+
+            if sess_filter and assr.get_session_label() not in sess_filter:
                 LOGGER.debug('ignoring:' + t)
                 continue
 
         LOGGER.debug('loading:' + t)
-        task = ClusterTask(os.path.splitext(t)[0], results_dir, diskq_dir)
+        task = ClusterTask(os.path.splitext(t)[0], resdir, diskq_dir)
         LOGGER.debug('status = ' + task.get_status())
 
         if not status or task.get_status() == status:
