@@ -75,7 +75,6 @@ OPEN_QA_LIST = [RERUN, REPROC]
 BAD_QA_STATUS = [FAILED, BAD, POOR, DONOTRUN]
 
 # Other Constants
-RESULTS_DIR = DAX_SETTINGS.get_results_dir()
 DEFAULT_EMAIL_OPTS = DAX_SETTINGS.get_email_opts()
 JOB_EXTENSION_FILE = DAX_SETTINGS.get_job_extension_file()
 READY_TO_UPLOAD_FLAG_FILENAME = 'READY_TO_UPLOAD.txt'
@@ -461,7 +460,7 @@ undo_processing...')
 
     def launch(self, jobdir, job_email=None,
                job_email_options=DAX_SETTINGS.get_email_opts(),
-               xnat_host=None, writeonly=False, pbsdir=None,
+               job_rungroup=None,xnat_host=None, writeonly=False, pbsdir=None,
                force_no_qsub=False):
         """
         Method to launch a job on the grid
@@ -470,6 +469,7 @@ undo_processing...')
         :param job_email: who to email if the job fails
         :param job_email_options: grid-specific job email options (e.g.,
          fails, starts, exits etc)
+        :param job_rungroup: grid-specific group to run the job under
         :param xnat_host: set the XNAT_HOST in the PBS job
         :param writeonly: write the job files without submitting them
         :param pbsdir: folder to store the pbs file
@@ -484,10 +484,17 @@ undo_processing...')
         outlog = self.outlog_path()
         outlog_dir = os.path.dirname(outlog)
         mkdirp(outlog_dir)
-        pbs = PBS(pbsfile, outlog, cmds, self.processor.walltime_str,
-                  self.processor.memreq_mb, self.processor.ppn,
-                  self.processor.env, job_email,
-                  job_email_options, xnat_host,
+        pbs = PBS(pbsfile,
+                  outlog,
+                  cmds,
+                  self.processor.walltime_str,
+                  self.processor.memreq_mb,
+                  self.processor.ppn,
+                  self.processor.env,
+                  job_email,
+                  job_email_options,
+                  job_rungroup,
+                  xnat_host,
                   self.processor.job_template)
         pbs.write()
         if writeonly:
@@ -499,6 +506,8 @@ undo_processing...')
                                            force_no_qsub=force_no_qsub)
 
             if jobid == '' or jobid == '0':
+                # TODO: check to be sure it didn't really launch, then
+                # try one more time
                 LOGGER.error('failed to launch job on cluster')
                 raise ClusterLaunchException
             else:
@@ -756,7 +765,7 @@ undo_processing...')
          be submitted to the scheduler for execution.
 
         """
-        res_dir = os.path.join(DAX_SETTINGS.get_results_dir())
+        res_dir = self.upload_dir
         j_ext = DAX_SETTINGS.get_job_extension_file()
         assessor_label = assessor_utils.full_label_from_assessor(
             self.assessor)
@@ -767,7 +776,10 @@ undo_processing...')
             else:
                 return os.path.join(os.path.join(res_dir, 'TRASH'), filename)
         else:
-            return os.path.join(os.path.join(res_dir, 'PBS'), filename)
+            if pbsdir:
+                return os.path.join(pbsdir, filename)
+            else:
+                return os.path.join(os.path.join(res_dir, 'PBS'), filename)
 
     def outlog_path(self):
         """
@@ -1354,7 +1366,8 @@ class ClusterTask(Task):
             shutil.move(src, dst)
 
         # Touch file for dax_upload to check
-        create_flag(os.path.join(RESULTS_DIR, self.assessor_label,
+        res_dir = self.upload_dir
+        create_flag(os.path.join(res_dir, self.assessor_label,
                     '%s.txt' % READY_TO_COMPLETE))
 
         return COMPLETE
@@ -1378,11 +1391,12 @@ class ClusterTask(Task):
         shutil.move(src, dst)
 
         # Touch file for dax_upload that job failed
-        create_flag(os.path.join(RESULTS_DIR, self.assessor_label,
+        res_dir = self.upload_dir
+        create_flag(os.path.join(res_dir, self.assessor_label,
                     '%s.txt' % JOB_FAILED))
 
         # Touch file for dax_upload to check
-        create_flag(os.path.join(RESULTS_DIR, self.assessor_label,
+        create_flag(os.path.join(res_dir, self.assessor_label,
                     '%s.txt' % READY_TO_COMPLETE))
 
         return JOB_FAILED
@@ -1529,13 +1543,15 @@ undo_processing...')
     def build_task(self, assr, sessions,
                    jobdir, job_email=None,
                    job_email_options=DEFAULT_EMAIL_OPTS,
+                   job_rungroup=None,
                    xnat_host=None):
         """
         Method to build a job
         """
         (old_proc_status, old_qc_status, _) = self.get_statuses(sessions)
         try:
-            cmds = self.build_commands(assr, sessions, jobdir)
+            resdir = self.upload_dir
+            cmds = self.build_commands(assr, sessions, jobdir, resdir)
             batch_file = self.batch_path()
             outlog = self.outlog_path()
             batch = PBS(batch_file,
@@ -1547,6 +1563,7 @@ undo_processing...')
                         self.processor.env,
                         job_email,
                         job_email_options,
+                        job_rungroup,
                         xnat_host,
                         self.processor.job_template)
             LOGGER.info('writing:' + batch_file)
@@ -1567,7 +1584,7 @@ undo_processing...')
 
         return (new_proc_status, new_qc_status)
 
-    def build_commands(self, assr, sessions, jobdir):
+    def build_commands(self, assr, sessions, jobdir, resdir):
         """
         Call the build_cmds method of the class Processor.
 
@@ -1581,4 +1598,5 @@ undo_processing...')
             assr,
             self.assessor_label,
             sessions,
-            jobdir)
+            jobdir,
+            resdir)
